---
title: Documentation
summary: Rules that are related to code documentation.
permalink: pmd_rules_apex_documentation.html
folder: pmd/rules/apex
sidebaractiveurl: /pmd_rules_apex.html
editmepath: ../pmd-apex/src/main/resources/category/apex/documentation.xml
keywords: Documentation, ApexDoc
language: Apex
---
<!-- DO NOT EDIT THIS FILE. This file is generated from file ../pmd-apex/src/main/resources/category/apex/documentation.xml. -->
## ApexDoc

**Since:** PMD 6.8.0

**Priority:** Medium (3)

This rule validates that:

* ApexDoc comments are present for classes, methods, and properties that are public or global, excluding
overrides and test classes (as well as the contents of test classes).
* ApexDoc comments should contain @description.
* ApexDoc comments on non-void, non-constructor methods should contain @return.
* ApexDoc comments on void or constructor methods should not contain @return.
* ApexDoc comments on methods with parameters should contain @param for each parameter, in the same
order as the method signature.

Method overrides and tests are both exempted from having ApexDoc.

**This rule is defined by the following Java class:** [net.sourceforge.pmd.lang.apex.rule.documentation.ApexDocRule](https://github.com/pmd/pmd/blob/master/pmd-apex/src/main/java/net/sourceforge/pmd/lang/apex/rule/documentation/ApexDocRule.java)

**Example(s):**

``` java
/**
 * @description Hello World
 */
public class HelloWorld {
    /**
     * @description Bar
     * @return Bar
     */
    public Object bar() { return null; }
}
```

**This rule has the following properties:**

|Name|Default Value|Description|Multivalued|
|----|-------------|-----------|-----------|
|cc\_categories|Style|Code Climate Categories|yes. Delimiter is '\|'.|
|cc\_remediation\_points\_multiplier|1|Code Climate Remediation Points multiplier|no|
|cc\_block\_highlighting|false|Code Climate Block Highlighting|no|

**Use this rule by referencing it:**
``` xml
<rule ref="category/apex/documentation.xml/ApexDoc" />
```

